const std  = @import("std");
const dvui = @import("dvui");

const ui = @import("../ui/root.zig");

pub const Task = struct {
    id: usize,
    name: [:0]const u8,
};

pub fn gui(ctx: anytype) !void {
    var win = try dvui.floatingWindow(@src(), .{}, .{});
    defer win.deinit();

    const cw = dvui.currentWindow();

    try dvui.windowHeader("Search", "", null);

    // task adder
    {
        var adder = try dvui.textEntry(@src(), .{}, .{
            .expand = .horizontal,
        });
        defer adder.deinit();

        if (dvui.focusedWidgetId() == adder.data().id) {
            if (ui.event.keyPress("activate")) {
                std.debug.print("hello\n", .{});
            }
        }
    }

    // task list
    {
        var scrollArea = try dvui.scrollArea(@src(), .{}, .{
            .expand = .both,
        });
        defer scrollArea.deinit();

        var list = try dvui.box(@src(), .vertical, .{
            .expand  = .both,
            .padding = dvui.Rect.all(4),
        });
        defer list.deinit();

        for (0.., ctx.tasks()) |idx, task| {
            if (idx > 0) {
                _ = try dvui.spacer(@src(), .{ .w = 0, .h = 1}, .{
                    .id_extra = idx,
                    .expand   = .horizontal,
                });
            }
            var hbox = try dvui.box(@src(), .horizontal, .{
                .id_extra   = idx,
                .expand     = .horizontal,
                .background = true,
                .border     = dvui.Rect.all(1),
            });
            defer hbox.deinit();

            try dvui.label(@src(), "#{d} {s}", .{ task.id, task.name, }, .{});

            if (hbox.data().borderRect().contains(cw.mouse_pt)) {
                var icons = try dvui.box(@src(), .horizontal, .{
                    .expand    = .vertical,
                    .gravity_x = 1,
                });
                defer icons.deinit();

                dvui.
            }
        }

        try dvui.labelNoFmt(@src(), "no tasks anymore :)", .{ .gravity_x = 0.5 });
    }
}

pub const Mock = struct {
    const Self     = @This();
    const TaskList = std.ArrayList(Task);

    _tasks: TaskList,

    pub fn init(alloc: std.mem.Allocator) !Self {
        var ts = TaskList.init(alloc);
        errdefer ts.deinit();

        try ts.append(Task { .id = 0, .name = "helloworld", });
        try ts.append(Task { .id = 1, .name = "goodbye", });

        return Mock {
            ._tasks = ts,
        };
    }
    pub fn deinit(self: *Self) void {
        self._tasks.deinit();
    }

    fn tasks(self: *const Self) []const Task {
        return self._tasks.items;
    }
};
